package HFE;

import java.lang.*;
import java.io.*;
import java.util.*;
import java.security.*;
import java.security.interfaces.*;
import sun.misc.*;
import HFE.*;

//-----------------------------------------------------------------------------
// Project:     Implementing QuartzLight in Java
//-----------------------------------------------------------------------------

/**
<p>
Holds speed test routines for QuartzLight
<p>
@author Christopher Wolf (hfe@Christopher-Wolf.de)
@version 0.15
*/

//Copyright (C) 2002    Christopher Wolf
//
//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License (file LICENCE) for more details.


class SpeedTest {
  /** Tests how fast message signature runs on the current hardware.
  */
  public final static void speedSignatureSHA_1() {
    //Check.assert(false);  // make sure we do not have any assertion checkers
    long overallTime = System.currentTimeMillis();
    long curTime;
    int paramM = 102;

    System.out.println("Start speed measurement SHA-1...");

    /**/
    System.gc();
    // generate a message digest with SHA-1
    try {
      String plaintext;       // plaintext for message digest
      // ...... generate suitable SHA-1 object
      MessageDigest md = MessageDigest.getInstance("SHA-1");

      System.out.println("n=" + paramM);
      System.out.println("r=10000");
      System.out.println("Message Digest SHA-1");
      for (int i = 0; i < paramM; i++) {
        String plainHash = " 123456789 123456789";
        curTime = System.currentTimeMillis();
        for (int r = 0; r < 10000; r++) {
          byte[] data = plainHash.getBytes("UTF8");
          byte[] hashData = md.digest(data);
        }
        curTime = System.currentTimeMillis() - curTime;
        System.out.println(curTime + " ms");
      }
    }
    catch(IOException ioe) {
      ioe.printStackTrace();
      return;
    }
    catch(NoSuchAlgorithmException nsae) {
      nsae.printStackTrace();
      System.out.println("\n\nMAJOR ERROR - SHA-1 Algorithm not available, Exiting...");
      System.exit(1);
    }

    System.out.println();
    System.gc();
    KeyPair pair;
    PrivateKey priv;
    PublicKey pub;
    // generate the public/private key pair for DSA
    try {
      String plaintext;       // plaintext for message digest
      // ...... generate suitable SHA-1 object

      System.out.println("\nn=" + paramM);
      System.out.println("r=1");
      System.out.println("Key Generation SHA-1/DSA [1024 bits]");
      for (int i = 0; i < paramM; i++) {
        curTime = System.currentTimeMillis();
        /* Generate a key pair */

        KeyPairGenerator keyGenGen = KeyPairGenerator.getInstance("DSA");
        DSAKeyPairGenerator keyGenDSA = (DSAKeyPairGenerator) keyGenGen;
        SecureRandom rnd = new SecureRandom();
        keyGenDSA.initialize(1024, true, rnd);
        pair = keyGenGen.generateKeyPair();
        priv = pair.getPrivate();
        pub = pair.getPublic();
        Signature dsa = Signature.getInstance("SHA1withDSA");
        dsa.initSign(priv);

        curTime = System.currentTimeMillis() - curTime;
        System.out.println(curTime + " ms");
      }
    }
    catch(Exception e) {
      e.printStackTrace();
      System.exit(1);
    }


    byte[][] realSigs = new byte[paramM][];
    System.out.println();
    System.gc();
    // sign a message
    try {
      String plaintext;       // plaintext for message digest
      // ...... generate suitable SHA-1 object
      KeyPairGenerator keyGen = KeyPairGenerator.getInstance("DSA");
      pair = keyGen.generateKeyPair();
      priv = pair.getPrivate();
      pub = pair.getPublic();
      Signature dsa = Signature.getInstance("SHA1withDSA");
      dsa.initSign(priv);

      System.out.println("n=" + paramM);
      System.out.println("r=100");
      System.out.println("Signature Computation DSA/SHA-1 [1024 bits]");
      for (int i = 0; i < paramM; i++) {
        String plainHash = " 123456789 123456789" + i;
        curTime = System.currentTimeMillis();
        for (int r = 0; r < 100; r++) {
          byte[] data = plainHash.getBytes("UTF8");

          dsa.update(data);

          realSigs[i] = dsa.sign();
        }
        curTime = System.currentTimeMillis() - curTime;
        System.out.println(curTime + " ms");
      }
    }
    catch(Exception e) {
      e.printStackTrace();
      return;
    }

    System.out.println();
    System.gc();
    // sign a message
    try {
      String plaintext;       // plaintext for message digest
      // ...... generate suitable SHA-1 object
      boolean validBool = false;
      Signature dsa = Signature.getInstance("SHA1withDSA");
      dsa.initVerify(pub);

      System.out.println("n=" + paramM);
      System.out.println("r=100");
      System.out.println("Signature Verification DSA/SHA-1 [1024 bits]");
      for (int i = 0; i < paramM; i++) {
        String plainHash = " 123456789 123456789" + i;
        curTime = System.currentTimeMillis();
        for (int r = 0; r < 100; r++) {
          byte[] data = plainHash.getBytes("UTF8");

          dsa.update(data);

          validBool = dsa.verify(realSigs[i]);
        }
        curTime = System.currentTimeMillis() - curTime;
        System.out.println(curTime + " ms");
        if (!validBool) System.out.println("Problem with signature verification.");
      }
    }
    catch(Exception e) {
      e.printStackTrace();
      return;
    }

    System.out.println();
    System.out.println("n=0");  // finis!
    System.out.println();
    long overall2 = System.currentTimeMillis() - overallTime;
    System.out.println("Overall time: " + overall2 + " ms (about " + (((float)overall2) / (1000*60))+
                       " min, i.e., " +(((float)overall2) / (1000*60*60))  + " hours)");
  }

  /** Has some speed measurements
  */
  public static void main(String[] args) {
    speedSignatureSHA_1();
  }
}
